
import os
from tests.test_base import TestBase
from capanno_utils.repo_config import config
from capanno_utils.validate import validate_tools_dir, validate_scripts_dir, validate_repo


class TestValidateDirectories(TestBase):

    def test_validate_tools_dir(self):
        validate_tools_dir(base_dir=config[os.environ['CONFIG_KEY']]['base_path'])
        return

    def test_validate_scripts_dir(self):
            validate_scripts_dir(base_dir=config[os.environ['CONFIG_KEY']]['base_path'])
            return

    # def test_validate_repo(self):
    #     validate_repo(base_dir=config[os.environ['CONFIG_KEY']]['base_path'])
    #     return